import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Main extends Frame implements ItemListener {
    ImageList list = new ImageList();

    Main(String[] filenames) {
        super("ItemListener Example");

        // Add images.
        for (int i=0; i<filenames.length; i++) {
            list.add(getToolkit().getImage(filenames[i]));
        }

        // Layout component and listen for item events.
        add(list, BorderLayout.CENTER);
        list.addItemListener(this);

        pack();
        show();
    }

    public void itemStateChanged(ItemEvent evt) {
        // Simply print out the event.
        System.out.println(evt);
    }

    public static void main(String args[]) {
        if (args.length < 1) {
            System.err.println(
                "Usage: java Main <imagefile 1> <imagefile 2>...");
            System.exit(1);
        }

        new Main(args);
    }
}

class ImageList extends Component implements ItemSelectable {
    // Number of pixels to surround each image.
    int margin = 3;

    // Currently selected image; -1 means no image selected.
    int curImage = -1;

    // Used to hold the images.
    Vector images = new Vector();

    ImageList() {
         addMouseListener(new MouseEventHandler());
    }

    // Adds an image to the image list.
    void add(Image image) {
        MediaTracker tracker = new MediaTracker(this);

        // We need to get the dimensions of the image.
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }
        images.addElement(image);
    }

    public Object[] getSelectedObjects() {
        if (curImage >= 0) {
            Object[] result = {images.elementAt(curImage)};

            return result;
        }
        return null;
    }

    // Paint the items.
    public void paint(Graphics g) {
        int y = margin;

        for (int i=0; i<images.size(); i++) {
            Image image = (Image)images.elementAt(i);
            int x = (getSize().width - image.getWidth(null))/2;

            g.drawImage(image, x, y, this);
            if (i == curImage) {
                // Draw an outline around selected image.
                g.setColor(Color.red);
                for (int j=1; j<=margin; j++) {
                    g.drawRect(x-j, y-j, 
                        image.getWidth(null)+2*j-1,
                        image.getHeight(null)+2*j-1);
                }
            }
            y += image.getHeight(null) + 2*margin;
        }
    }

    // Computes the preferred size of the whole image list.
    public Dimension getPreferredSize() {
        int width = 0;
        int height = 0;

        for (int i=0; i<images.size(); i++) {
            Image image = (Image)images.elementAt(i);

            width = Math.max(width, image.getWidth(null));
            height += image.getHeight(null);
        }

        // Add space for margins.
        width += 2*margin;
        height += 2*margin*(images.size());

        return new Dimension(width, height);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            int y = margin;

            // Find which image under the mouse coordinates.
            for (int i=0; i<images.size(); i++) {
                Image image = (Image)images.elementAt(i);
                int x = (getSize().width - image.getWidth(null))/2;

                // Is the mouse coordinates on this image?
                if (evt.getX() > x && evt.getX() < x+image.getWidth(null)
                  && evt.getY() > y && evt.getY() < y+image.getHeight(null)) {
                    // Fire an item event to deselect previous item.
                    if (curImage >= 0) {
                        processItemEvent(
                            new ItemEvent(ImageList.this, 
                                ItemEvent.ITEM_STATE_CHANGED,
                                new Integer(curImage), 
                                ItemEvent.DESELECTED));
                    }

                    // Now fire an item event to select new item.
                    curImage = i;
                    processItemEvent(
                        new ItemEvent(ImageList.this, 
                            ItemEvent.ITEM_STATE_CHANGED,
                            new Integer(curImage), 
                            ItemEvent.SELECTED));

                    repaint();
                    break;
                }
                y += image.getHeight(null) + 2*margin;
            }
        }
    }

    // Methods to support item event listeners.
    transient ItemListener itemListener;
    public synchronized void addItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }
    public synchronized void removeItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }
    protected void processItemEvent(ItemEvent e) {
        if (itemListener != null) {
            itemListener.itemStateChanged(e);
        }
    }
}
