import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements AdjustmentListener {
    Slider ver = new Slider(Adjustable.VERTICAL);
    Slider hor = new Slider(Adjustable.HORIZONTAL);
    
    float[] hsb = {1.0f, 0.1f, 0.7f};

    // This off-screen image is used for double-buffering.
    Image bbuf;
    
    Main() {
        super("Adjustable Example");

        ver.setMaximum(1000);
        hor.setMaximum(1000);

        // Register listeners.
        ver.addAdjustmentListener(this);
        hor.addAdjustmentListener(this);
    
        // Set up the frame
        setLayout(new BorderLayout());
        add(ver, BorderLayout.EAST);
        add(hor, BorderLayout.SOUTH);
        setSize(200, 200);
        setVisible(true);
    }

    // Updates the color of the oval.
    public void adjustmentValueChanged(AdjustmentEvent evt) {
        Adjustable a = evt.getAdjustable();

        if (a.getOrientation() == Adjustable.VERTICAL) {
            hsb[1] = (float)a.getValue()/(float)a.getMaximum();
        } else {
            hsb[0] = (float)a.getValue()/(float)a.getMaximum();
        }
        repaint();
    }
    
    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;
        Insets insets = getInsets();

        // Create or enlarge the double-buffer if necessary.
        if (bbuf == null 
              || bbuf.getWidth(this) < w
              || bbuf.getHeight(this) < h) {
            if (bbuf != null) {
                bbuf.flush();
            }
            bbuf = createImage(w, h);
        }
        Graphics bbufG = bbuf.getGraphics();

        bbufG.setColor(getBackground());
        bbufG.fillRect(0, 0, w, h);

        // Draw oval.
        bbufG.setColor(new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2])));
        bbufG.fillOval(insets.left, insets.top, 
            w-insets.right-insets.left -ver.getSize().width, 
            h-insets.bottom-insets.top -hor.getSize().height);

        // Paint double-buffer on screen.
        g.drawImage(bbuf, 0, 0, this);
        bbufG.dispose();
    }

    public static void main(String args[]) {
        new Main();
    }
}

class Slider extends Canvas implements Adjustable {
    // The location of the ball.  
    Point locPt = new Point(0, 0);

    // If non-null, the ball is being dragged and this field
    // holds the location of the cursor relative to the ball 
    // when the mouse was pressed.
    Point downPt;

    // The image of the ball.
    Image image;

    // Dimensions of the ball.
    int ballW, ballH;

    // Off-screen image is used for double-buffering.
    Image bbuf;
    
    Slider(int o) {
        orient = o;
        image = Toolkit.getDefaultToolkit().getImage("ball.gif");

        // Wait for the image to be loaded so that we
        // can determine its size.
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        try {
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }
         
        setSize(ballW = image.getWidth(this), 
            ballH = image.getHeight(this));

        // Register listeners.
        addMouseListener(new MouseEventListener());
        addMouseMotionListener(new MouseMotionEventListener());
        addKeyListener(new KeyEventListener());
    }

    public void invalidate() {
        super.invalidate();

        // The bounds of the slider has changed so recompute 
        // the ball location.
        setValues(value, min, max);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;

        // Create or enlarge the double-buffer if necessary.
        if (bbuf == null
              || bbuf.getWidth(this) < w
              || bbuf.getHeight(this) < h) {
            if (bbuf != null) {
                bbuf.flush();
            }
            bbuf = createImage(w, h);
        }
        Graphics bbufG = bbuf.getGraphics();

        // Clear the background.
        bbufG.setColor(getBackground());
        bbufG.fillRect(0, 0, w, h);

        // Draw the track.
        bbufG.setColor(Color.gray);
        if (orient == Adjustable.VERTICAL) {
            bbufG.fillRoundRect(w/2-1, ballH/2, 3, h - ballH, 5, 5);
        } else {
            bbufG.fillRoundRect(ballW/2, h/2-1, w - ballW, 3, 5, 5);
        }

        // Draw the ball.
        bbufG.drawImage(image, locPt.x, locPt.y, this);

        // Paint double-buffer on screen.
        g.drawImage(bbuf, 0, 0, this);
        bbufG.dispose();
    }

    // Returns -1 if above/left of ball;
    //          0 if on ball.
    //          1 if below/right of ball;
    int onBall(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();

        if (orient == Adjustable.VERTICAL) {
            if (y < locPt.y) {
                return -1;
            } else if (y > locPt.y+ballH) {
                return 1;
            }
        } else {
            if (x < locPt.x) {
                return -1;
            } else if (x > locPt.x+ballW) {
                return 1;
            }
        }
        return 0;
    }

    // Handle the events.
    class MouseEventListener extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            switch (onBall(evt)) {
              case -1:
                inc(AdjustmentEvent.BLOCK_DECREMENT, -blockInc);
                break;
              case 0:
                // Enter dragging mode.
                (downPt = evt.getPoint()).translate(-locPt.x, -locPt.y);
                break;
              case 1:
                inc(AdjustmentEvent.BLOCK_INCREMENT, blockInc);
                break;
            } 
            requestFocus();
        }

        public void mouseReleased(MouseEvent evt) {
            downPt = null;
        }

        public void mouseEntered(MouseEvent evt) {
            if (onBall(evt) == 0) {
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }
        }

        public void mouseExited(MouseEvent evt) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }

    class MouseMotionEventListener extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            if (downPt != null) {
                if (orient == Adjustable.VERTICAL) {
                    setValues(evt.getY() * (max-min) 
                        / (getSize().height-ballH) + min, min, max);
                } else {
                    setValues(evt.getX() * (max-min) 
                        / (getSize().width-ballW) + min, min, max);
                }
                inc(AdjustmentEvent.TRACK, 0);
            }
        }
        public void mouseMoved(MouseEvent evt) {
            if (onBall(evt) == 0) {
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            } 
        }
    }
    class KeyEventListener extends KeyAdapter {
        public void keyPressed(KeyEvent evt) {
            switch (evt.getKeyCode()) {
              case KeyEvent.VK_DOWN:
              case KeyEvent.VK_RIGHT:
                inc(AdjustmentEvent.UNIT_INCREMENT, unitInc);
                break;
              case KeyEvent.VK_PAGE_DOWN:
                inc(AdjustmentEvent.BLOCK_INCREMENT, blockInc);
                break;
              case KeyEvent.VK_UP:
              case KeyEvent.VK_LEFT:
                inc(AdjustmentEvent.UNIT_DECREMENT, -unitInc);
                break;
              case KeyEvent.VK_PAGE_UP:
                inc(AdjustmentEvent.BLOCK_DECREMENT, -blockInc);
                break;
            }
        }
    }


    // ***** Adjustable Get Methods *****

    // The properties.
    int orient;
    int min = 0;
    int max = 100;
    int unitInc = 1;
    int blockInc = 10;
    int visible = 0;
    int value = 0;

    public int getOrientation() {
        return orient;
    }

    public int getValue() {
        return value;
    }

    public int getMinimum() {
        return min;
    }

    public int getMaximum() {
        return max;
    }

    public int getVisibleAmount() {
        return 0;
    }

    public int getUnitIncrement() {
        return unitInc;
    }

    public int getBlockIncrement() {
        return blockInc;
    }

    // ***** Adjustable Set Methods *****

    void setValues(int va, int mn, int mx) {
        value = va;
        min = mn;
        max = mx;
        if (max <= min) {
            max = min + 1;
        }
        if (value < min) {
            value = min;
        }
        if (value > max) {
            value = max;
        }

        // Update the location of the ball.
        if (orient == Adjustable.VERTICAL) {
            locPt.y = (value-min) * (getSize().height-ballH) 
                / (max-min);
        } else {
            locPt.x = (value-min) * (getSize().width-ballW) 
                / (max-min);
        }

        // Generate an adjustment event.
        inc(AdjustmentEvent.TRACK, 0);
        repaint();
    }

    public void setValue(int v) {
        setValues(v, min, max);
    }

    public void setMinimum(int m) {
        setValues(value, m, max);
    }

    public void setMaximum(int m) {
        setValues(value, min, m);
    }

    public void setVisibleAmount(int v) {
    }

    public void setUnitIncrement(int u) {
        unitInc = u;
    }

    public void setBlockIncrement(int b) {
        blockInc = b;
    }

    // ***** Adjustable Event Methods *****
    
    protected void processEvent(AWTEvent evt) {
        if (evt instanceof AdjustmentEvent) {
            processAdjustmentEvent((AdjustmentEvent)evt);
            return;
        }
        super.processEvent(evt);
    }

    // Deliver event to listeners.
    protected void processAdjustmentEvent(AdjustmentEvent evt) {
        if (adjustmentListener != null) {
            adjustmentListener.adjustmentValueChanged(evt);
        }
    }

    void inc(int type, int inc) {
        if (inc != 0) {
            setValues(value + inc, min, max);
        }
        if (adjustmentListener != null) {
            processEvent(new AdjustmentEvent(this,
                AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, type, value));
        }
    }

    // ***** Adjustable Listener Methods *****

    transient AdjustmentListener adjustmentListener;

    public void addAdjustmentListener(AdjustmentListener l) {
        adjustmentListener = AWTEventMulticaster.add(adjustmentListener, l);
    }

    public void removeAdjustmentListener(AdjustmentListener l) {
        adjustmentListener = AWTEventMulticaster.remove(adjustmentListener, l);
    }

}
